@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -22,7 +22,7 @@ from bzrlib.bzrdir import BzrDirFormat, format_registry
 from bzrlib.commands import Command, register_command, display_command, Option
 from bzrlib.help_topics import topic_registry
 from bzrlib.revisionspec import SPEC_TYPES
-from bzrlib.trace import warning
+from bzrlib.trace import warning, mutter
 from bzrlib.transport import register_lazy_transport, register_transport_proto
 
 import format
@@ -31,7 +31,7 @@ import revspec
 # versions ending in 'exp' mean experimental mappings
 # versions ending in 'dev' mean development version
 # versions ending in 'final' mean release (well tested, etc)
-version_info = (0, 4, 9, 'final', 0)
+version_info = (0, 4, 10, 'final', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]
@@ -39,7 +39,7 @@ else:
     version_string = '%d.%d.%d%s%d' % version_info
 __version__ = version_string
 
-COMPATIBLE_BZR_VERSIONS = [(1, 3)]
+COMPATIBLE_BZR_VERSIONS = [(1, 4), (1, 5)]
 
 def check_bzrlib_version(desired):
     """Check that bzrlib is compatible.
@@ -79,6 +79,8 @@ def check_subversion_version():
         warning('Installed Subversion version does not have updated Python '
                 'bindings. See the bzr-svn README for details.')
         raise bzrlib.errors.BzrError("incompatible python subversion bindings")
+    import svn.core
+    mutter("bzr-svn: using Subversion %d.%d.%d (%s)" % (svn.core.SVN_VER_MAJOR, svn.core.SVN_VER_MINOR, svn.core.SVN_VER_MICRO, svn.core.__file__))
 
 check_subversion_version()
 
@@ -97,7 +99,7 @@ register_lazy_transport('svn://', 'bzrlib.plugins.svn.transport',
 register_lazy_transport('svn+', 'bzrlib.plugins.svn.transport', 
                         'SvnRaTransport')
 topic_registry.register_lazy('svn-branching-schemes', 
-                             'bzrlib.plugins.svn.scheme',
+                             'bzrlib.plugins.svn.mapping3.scheme',
                              'help_schemes', 'Subversion branching schemes')
 
 BzrDirFormat.register_control_format(format.SvnRemoteFormat)
@@ -145,7 +147,7 @@ def get_scheme(schemename):
     """
     if isinstance(schemename, unicode):
         schemename = schemename.encode("ascii")
-    from scheme import BranchingScheme
+    from mapping3.scheme import BranchingScheme
     from bzrlib.errors import BzrCommandError
     
     ret = BranchingScheme.find_scheme(schemename)
@@ -223,13 +225,13 @@ class cmd_svn_import(Command):
             raise BzrCommandError(
                     "Not a Subversion repository: %s" % from_location)
 
-        def filter_branch((branch_path, revnum, exists)):
-            if prefix is not None and not branch_path.startswith(prefix):
+        def filter_branch(branch):
+            if prefix is not None and not branch.get_branch_path().startswith(prefix):
                 return False
-            return exists
+            return True
 
-        convert_repository(from_repos, to_location, scheme, not standalone, 
-                trees, all, filter_branch=filter_branch)
+        convert_repository(from_repos, to_location, scheme, None, 
+                           not standalone, trees, all, filter_branch=filter_branch)
 
         if tmp_repos is not None:
             from bzrlib import osutils
@@ -339,7 +341,11 @@ class cmd_svn_push(Command):
             revision_id = None
         try:
             target_branch = bzrdir.open_branch()
-            target_branch.pull(source_branch, revision_id)
+            target_branch.lock_write()
+            try:
+                target_branch.pull(source_branch, revision_id)
+            finally:
+                target_branch.unlock()
         except NotBranchError:
             target_branch = bzrdir.import_branch(source_branch, revision_id)
         # We successfully created the target, remember it
@@ -368,7 +374,7 @@ class cmd_svn_branching_scheme(Command):
         from bzrlib.repository import Repository
         from bzrlib.trace import info
         from repository import SvnRepository
-        from scheme import scheme_from_branch_list
+        from mapping3.scheme import scheme_from_branch_list
         def scheme_str(scheme):
             if scheme is None:
                 return ""
@@ -380,16 +386,16 @@ class cmd_svn_branching_scheme(Command):
         if repository_wide:
             scheme = repos._get_property_scheme()
         else:
-            scheme = repos.get_scheme()
+            scheme = repos.get_mapping().scheme
         if set:
             schemestr = edit_commit_message("", 
                                             start_message=scheme_str(scheme))
             scheme = scheme_from_branch_list(
                 map(lambda x:x.strip("\n"), schemestr.splitlines()))
             if repository_wide:
-                repos.set_property_scheme(scheme)
+                set_property_scheme(repos, scheme)
             else:
-                repos.set_branching_scheme(scheme, mandatory=True)
+                set_config_scheme(repos, scheme, mandatory=True)
         elif scheme is not None:
             info(scheme_str(scheme))
 
