@@ -17,6 +17,7 @@
 
 from bzrlib.config import AuthenticationConfig
 from bzrlib.ui import ui_factory
+import svn.core
 from svn.core import (svn_auth_cred_username_t, 
                       svn_auth_cred_simple_t,
                       svn_auth_cred_ssl_client_cert_t,
@@ -25,28 +26,35 @@ from svn.core import (svn_auth_cred_username_t,
                       svn_auth_get_username_prompt_provider,
                       svn_auth_get_simple_prompt_provider,
                       svn_auth_get_ssl_server_trust_prompt_provider,
-                      svn_auth_get_ssl_client_cert_pw_prompt_provider)
+                      svn_auth_get_ssl_client_cert_pw_prompt_provider,
+                      svn_auth_open)
+import urlparse
+import urllib
 
 
 class SubversionAuthenticationConfig(AuthenticationConfig):
     """Simple extended version of AuthenticationConfig that can provide 
     the information Subversion requires.
     """
-    def __init__(self, file=None, scheme="svn", host=None):
+    def __init__(self, scheme, host, port, path, file=None):
         super(SubversionAuthenticationConfig, self).__init__(file)
         self.scheme = scheme
         self.host = host
-
+        self.port = port
+        self.path = path
+       
     def get_svn_username(self, realm, may_save, pool=None):
         """Look up a Subversion user name in the Bazaar authentication cache.
 
         :param realm: Authentication realm (optional)
         :param may_save: Whether or not the username should be saved.
         :param pool: Allocation pool, is ignored.
+        :param default: Assumed username
         """
         username_cred = svn_auth_cred_username_t()
         username_cred.username = self.get_user(self.scheme, 
-                host=self.host, realm=realm)
+                host=self.host, path=self.path, 
+                realm=realm)
         username_cred.may_save = False
         return username_cred
 
@@ -60,16 +68,16 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         :param pool: Allocation pool, is ignored.
         """
         simple_cred = svn_auth_cred_simple_t()
-        simple_cred.username = username or self.get_username(realm, may_save, 
-                                             pool, prompt="%s password" % realm)
+        simple_cred.username = self.get_user(self.scheme, 
+                host=self.host, path=self.path, realm=realm) or username
         simple_cred.password = self.get_password(self.scheme, host=self.host, 
-                                    user=simple_cred.username, realm=realm,
-                                    prompt="%s password" % realm)
+            path=self.path, user=simple_cred.username, 
+            realm=realm, prompt="%s %s password" % (realm, simple_cred.username))
         simple_cred.may_save = False
         return simple_cred
 
     def get_svn_ssl_server_trust(self, realm, failures, cert_info, may_save, 
-                                 pool):
+                                     pool):
         """Return a Subversion auth provider that verifies SSL server trust.
 
         :param realm: Realm name (optional)
@@ -113,8 +121,7 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
     def get_svn_ssl_server_trust_prompt_provider(self):
         """Return a Subversion auth provider for checking 
         whether a SSL server is trusted."""
-        return svn_auth_get_ssl_server_trust_prompt_provider(
-                    self.get_svn_ssl_server_trust)
+        return svn_auth_get_ssl_server_trust_prompt_provider(self.get_svn_ssl_server_trust)
 
     def get_svn_auth_providers(self):
         """Return a list of auth providers for this authentication file.
@@ -123,7 +130,6 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
                 self.get_svn_simple_prompt_provider(1),
                 self.get_svn_ssl_server_trust_prompt_provider()]
 
-
 def get_ssl_client_cert_pw(realm, may_save, pool):
     """Simple SSL client certificate password prompter.
 
@@ -141,3 +147,51 @@ def get_ssl_client_cert_pw_provider(tries):
     return svn_auth_get_ssl_client_cert_pw_prompt_provider(
                 get_ssl_client_cert_pw, tries)
 
+
+def get_stock_svn_providers():
+    providers = [svn.client.get_simple_provider(),
+            svn.client.get_username_provider(),
+            svn.client.get_ssl_client_cert_file_provider(),
+            svn.client.get_ssl_client_cert_pw_file_provider(),
+            svn.client.get_ssl_server_trust_file_provider(),
+            ]
+
+    if hasattr(svn.client, 'get_windows_simple_provider'):
+        providers.append(svn.client.get_windows_simple_provider())
+
+    if hasattr(svn.client, 'get_keychain_simple_provider'):
+        providers.append(svn.client.get_keychain_simple_provider())
+
+    if hasattr(svn.client, 'get_windows_ssl_server_trust_provider'):
+        providers.append(svn.client.get_windows_ssl_server_trust_provider())
+
+    return providers
+
+
+def create_auth_baton(url):
+    """Create an authentication baton for the specified URL."""
+    assert isinstance(url, str)
+    (scheme, netloc, path, _, _) = urlparse.urlsplit(url)
+    (creds, host) = urllib.splituser(netloc)
+    (host, port) = urllib.splitport(host)
+
+    auth_config = SubversionAuthenticationConfig(scheme, host, port, path)
+
+    # Specify Subversion providers first, because they use file data
+    # rather than prompting the user.
+    providers = get_stock_svn_providers()
+
+    if svn.core.SVN_VER_MAJOR == 1 and svn.core.SVN_VER_MINOR >= 5:
+        providers += auth_config.get_svn_auth_providers()
+        providers += [get_ssl_client_cert_pw_provider(1)]
+
+    auth_baton = svn.core.svn_auth_open(providers)
+    if creds is not None:
+        (auth_baton.user, auth_baton.password) = urllib.splitpasswd(creds)
+        if auth_baton.user is not None:
+            svn.core.svn_auth_set_parameter(auth_baton, 
+                svn.core.SVN_AUTH_PARAM_DEFAULT_USERNAME, auth_baton.user)
+        if auth_baton.password is not None:
+            svn.core.svn_auth_set_parameter(auth_baton, 
+                svn.core.SVN_AUTH_PARAM_DEFAULT_PASSWORD, auth_baton.password)
+    return auth_baton
