@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -21,13 +21,14 @@ from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (NoSuchFile, DivergedBranches, NoSuchRevision, 
                            NotBranchError)
 from bzrlib.inventory import (Inventory)
-from bzrlib.revision import ensure_null, NULL_REVISION
+from bzrlib.revision import is_null, ensure_null, NULL_REVISION
 from bzrlib.workingtree import WorkingTree
 
 import svn.client, svn.core
-from svn.core import SubversionException, Pool
+from svn.core import SubversionException
 
 from commit import push
+from config import BranchConfig
 from errors import NotSvnBranchPath
 from format import get_rich_root_format
 from repository import SvnRepository
@@ -68,22 +69,23 @@ class SvnBranch(Branch):
         self._format = SvnBranchFormat()
         self._lock_mode = None
         self._lock_count = 0
-        self._cached_revnum = None
-        self._revision_history = None
-        self._revision_history_revnum = None
         self.mapping = self.repository.get_mapping()
         self._branch_path = branch_path.strip("/")
+        self._revmeta_cache = None
         assert isinstance(self._branch_path, str)
         try:
+            revnum = self.get_revnum()
+            if revnum is None:
+                raise NotBranchError(self.base)
             if self.repository.transport.check_path(branch_path.strip("/"), 
-                self.get_revnum()) != svn.core.svn_node_dir:
+                revnum) != svn.core.svn_node_dir:
                 raise NotBranchError(self.base)
         except SubversionException, (_, num):
             if num == svn.core.SVN_ERR_FS_NO_SUCH_REVISION:
                 raise NotBranchError(self.base)
             raise
         if not self.mapping.is_branch(branch_path):
-            raise NotSvnBranchPath(branch_path, scheme=self.mapping.scheme)
+            raise NotSvnBranchPath(branch_path, mapping=self.mapping)
 
     def set_branch_path(self, branch_path):
         """Change the branch path for this branch.
@@ -92,6 +94,10 @@ class SvnBranch(Branch):
         """
         self._branch_path = branch_path.strip("/")
 
+    def _get_append_revisions_only(self):
+        value = self.get_config().get_user_option('append_revisions_only')
+        return value == 'True'
+
     def unprefix(self, relpath):
         """Remove the branch path from a relpath.
 
@@ -119,7 +125,8 @@ class SvnBranch(Branch):
         """
         if self._lock_mode == 'r' and self._cached_revnum:
             return self._cached_revnum
-        self._cached_revnum = self.repository.transport.get_latest_revnum()
+        latest_revnum = self.repository.get_latest_revnum()
+        self._cached_revnum = self.repository._log.find_latest_change(self.get_branch_path(), latest_revnum)
         return self._cached_revnum
 
     def check(self):
@@ -155,7 +162,7 @@ class SvnBranch(Branch):
         :raises NoSuchRevision: If the revision id was not found.
         """
         (bp, revnum, mapping) = self.repository.lookup_revision_id(revid, 
-                                                             scheme=self.mapping.scheme)
+                                         ancestry=(self.get_branch_path(), self.get_revnum()))
         assert bp.strip("/") == self.get_branch_path(revnum).strip("/"), \
                 "Got %r, expected %r" % (bp, self.get_branch_path(revnum))
         return revnum
@@ -178,8 +185,9 @@ class SvnBranch(Branch):
             rev.kind = svn.core.svn_opt_revision_number
             rev.value.number = revnum
 
-        client_ctx = create_svn_client(Pool())
-        svn.client.checkout(bzr_to_svn_url(self.base), to_location, rev, 
+        svn_url = bzr_to_svn_url(self.base)
+        client_ctx = create_svn_client(svn_url)
+        svn.client.checkout(svn_url, to_location, rev, 
                             True, client_ctx)
 
         return WorkingTree.open(to_location)
@@ -197,13 +205,10 @@ class SvnBranch(Branch):
         assert isinstance(revnum, int)
         return self.repository.generate_revision_id(
                 revnum, self.get_branch_path(revnum), self.mapping)
-       
-    def _generate_revision_history(self, last_revnum):
-        """Generate the revision history up until a specified revision."""
-        revhistory = list(self.repository.iter_lhs_ancestry(self.generate_revision_id(last_revnum)))
-        revhistory.reverse()
-        return revhistory
 
+    def get_config(self):
+        return BranchConfig(self)
+       
     def _get_nick(self):
         """Find the nick name for this branch.
 
@@ -228,6 +233,16 @@ class SvnBranch(Branch):
         last_revid = self.last_revision()
         return self.revision_id_to_revno(last_revid), last_revid
 
+    def revision_id_to_revno(self, revision_id):
+        """Given a revision id, return its revno"""
+        if is_null(revision_id):
+            return 0
+        revmeta_history = self._revision_meta_history()
+        for revmeta in revmeta_history:
+            if revmeta.get_revision_id(self.mapping) == revision_id:
+                return len(revmeta_history) - revmeta_history.index(revmeta)
+        raise NoSuchRevision(self, revision_id)
+
     def get_root_id(self, revnum=None):
         if revnum is None:
             tree = self.basis_tree()
@@ -235,23 +250,6 @@ class SvnBranch(Branch):
             tree = self.repository.revision_tree(self.get_rev_id(revnum))
         return tree.get_root_id()
 
-    def revno(self):
-        """See Branch.revno()."""
-        return self.last_revision_info()[0]
-
-    def revision_id_to_revno(self, revision_id):
-        """See Branch.revision_id_to_revno()."""
-        if revision_id is None:
-            return 0
-        revno = self.repository.revmap.lookup_dist_to_origin(revision_id)
-        if revno is not None:
-            return revno
-        history = self.revision_history()
-        try:
-            return history.index(revision_id) + 1
-        except ValueError:
-            raise NoSuchRevision(self, revision_id)
-
     def set_push_location(self, location):
         """See Branch.set_push_location()."""
         raise NotImplementedError(self.set_push_location)
@@ -261,35 +259,33 @@ class SvnBranch(Branch):
         # get_push_location not supported on Subversion
         return None
 
-    def revision_history(self, last_revnum=None):
-        """See Branch.revision_history()."""
-        if last_revnum is None:
-            last_revnum = self.get_revnum()
-        if (self._revision_history is None or 
-            self._revision_history_revnum != last_revnum):
-            self._revision_history = self._generate_revision_history(last_revnum)
-            self._revision_history_revnum = last_revnum
-            self.repository.revmap.insert_revision_history(self._revision_history)
-        return self._revision_history
+    def _revision_meta_history(self):
+        if self._revmeta_cache is None:
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self._revmeta_cache = list(self.repository.iter_reverse_branch_changes(self.get_branch_path(), self.get_revnum(), self.mapping, pb=pb))
+            finally:
+                pb.finished()
+        return self._revmeta_cache
+
+    def _gen_revision_history(self):
+        """Generate the revision history from last revision
+        """
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            history = []
+            for revmeta in self._revision_meta_history():
+                history.append(revmeta.get_revision_id(self.mapping))
+        finally:
+            pb.finished()
+        history.reverse()
+        return history
 
     def last_revision(self):
         """See Branch.last_revision()."""
         # Shortcut for finding the tip. This avoids expensive generation time
         # on large branches.
-        last_revnum = self.get_revnum()
-        if (self._revision_history is None or 
-            self._revision_history_revnum != last_revnum):
-            for (branch, rev) in self.repository.follow_branch(
-                self.get_branch_path(), last_revnum, self.mapping):
-                return self.repository.generate_revision_id(rev, branch, 
-                                                            self.mapping)
-            return NULL_REVISION
-
-        ph = self.revision_history(last_revnum)
-        if ph:
-            return ph[-1]
-        else:
-            return NULL_REVISION
+        return self.generate_revision_id(self.get_revnum())
 
     def pull(self, source, overwrite=False, stop_revision=None, 
              _hook_master=None, run_hooks=True):
@@ -365,6 +361,7 @@ class SvnBranch(Branch):
                 pb.update("pushing revisions", todo.index(revid), 
                           len(todo))
                 push(self, other, revid)
+                self._clear_cached_state()
         finally:
             pb.finished()
 
@@ -377,6 +374,7 @@ class SvnBranch(Branch):
         else:
             self._lock_mode = 'w'
             self._lock_count = 1
+        self.repository.lock_write()
         
     def lock_read(self):
         """See Branch.lock_read()."""
@@ -386,13 +384,20 @@ class SvnBranch(Branch):
         else:
             self._lock_mode = 'r'
             self._lock_count = 1
+        self.repository.lock_read()
 
     def unlock(self):
         """See Branch.unlock()."""
         self._lock_count -= 1
         if self._lock_count == 0:
             self._lock_mode = None
-            self._cached_revnum = None
+            self._clear_cached_state()
+        self.repository.unlock()
+
+    def _clear_cached_state(self):
+        super(SvnBranch,self)._clear_cached_state()
+        self._cached_revnum = None
+        self._revmeta_cache = None
 
     def get_parent(self):
         """See Branch.get_parent()."""
@@ -403,6 +408,7 @@ class SvnBranch(Branch):
 
     def append_revision(self, *revision_ids):
         """See Branch.append_revision()."""
+        self._clear_cached_state()
         #raise NotImplementedError(self.append_revision)
         #FIXME: Make sure the appended revision is already 
         # part of the revision history
