@@ -2,7 +2,7 @@
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -80,7 +80,7 @@ def load_dumpfile(dumpfile, outputdir):
     return repos
 
 
-def convert_repository(source_repos, output_url, scheme=None, 
+def convert_repository(source_repos, output_url, scheme=None, layout=None,
                        create_shared_repo=True, working_trees=False, all=False,
                        format=None, filter_branch=None):
     """Convert a Subversion repository and its' branches to a 
@@ -88,13 +88,15 @@ def convert_repository(source_repos, output_url, scheme=None,
 
     :param source_repos: Subversion repository
     :param output_url: URL to write Bazaar repository to.
-    :param scheme: Branching scheme (object) to use
+    :param scheme: Branching scheme to use.
+    :param layout: Repository layout (object) to use
     :param create_shared_repo: Whether to create a shared Bazaar repository
     :param working_trees: Whether to create working trees
     :param all: Whether old revisions, even those not part of any existing 
         branches, should be imported
     :param format: Format to use
     """
+    from mapping3 import SchemeDerivedLayout, set_branching_scheme
     assert not all or create_shared_repo
     if format is None:
         format = get_rich_root_format()
@@ -111,30 +113,25 @@ def convert_repository(source_repos, output_url, scheme=None,
             dirs[path] = format.initialize_on_transport(nt)
         return dirs[path]
 
-    if scheme is not None:
-        source_repos.set_branching_scheme(scheme)
+    if layout is not None:
+        source_repos.set_layout(layout)
+    elif scheme is not None:
+        set_branching_scheme(source_repos, scheme)
+        layout = SchemeDerivedLayout(source_repos, scheme)
+    else:
+        layout = source_repos.get_layout()
 
     if create_shared_repo:
         try:
             target_repos = get_dir("").open_repository()
-            assert (source_repos.get_scheme().is_branch("") or 
-                    source_repos.get_scheme().is_tag("") or 
-                    target_repos.is_shared())
+            assert (layout.is_branch("") or layout.is_tag("") or target_repos.is_shared())
         except NoRepositoryPresent:
             target_repos = get_dir("").create_repository(shared=True)
         target_repos.set_make_working_trees(working_trees)
 
-    if filter_branch is None:
-        filter_branch = lambda (bp, rev, exists): exists
-
-    existing_branches = [(bp, revnum) for (bp, revnum, _) in 
-            filter(filter_branch,
-                   source_repos.find_branchpaths(source_repos.get_scheme()))]
-
-    def is_dir((branch, revnum)):
-        return source_repos.transport.check_path(branch, revnum) == svn.core.svn_node_dir
-
-    existing_branches = filter(is_dir, existing_branches)
+    existing_branches = source_repos.find_branches(layout=layout)
+    if filter_branch is not None:
+        existing_branches = filter(filter_branch, existing_branches)
 
     if create_shared_repo:
         inter = InterRepository.get(source_repos, target_repos)
@@ -144,32 +141,26 @@ def convert_repository(source_repos, output_url, scheme=None,
         elif (target_repos.is_shared() and 
               hasattr(inter, '_supports_branches') and 
               inter._supports_branches):
-            inter.fetch(branches=[source_repos.generate_revision_id(revnum, branch, source_repos.get_mapping()) for (branch, revnum) in existing_branches])
-
+            inter.fetch(branches=[branch.last_revision() for branch in existing_branches])
 
     source_graph = source_repos.get_graph()
     pb = ui.ui_factory.nested_progress_bar()
     try:
         i = 0
-        for (branch, revnum) in existing_branches:
-            pb.update("%s:%d" % (branch, revnum), i, len(existing_branches))
-            revid = source_repos.generate_revision_id(revnum, branch, 
-                                          source_repos.get_mapping())
-
-            target_dir = get_dir(branch)
+        for source_branch in existing_branches:
+            pb.update("%s:%d" % (source_branch.get_branch_path(), source_branch.get_revnum()), i, len(existing_branches))
+            target_dir = get_dir(source_branch.get_branch_path())
             if not create_shared_repo:
                 try:
                     target_dir.open_repository()
                 except NoRepositoryPresent:
                     target_dir.create_repository()
-            source_branch_url = urlutils.join(source_repos.base, branch)
             try:
                 target_branch = target_dir.open_branch()
             except NotBranchError:
                 target_branch = target_dir.create_branch()
-                target_branch.set_parent(source_branch_url)
-            if revid != target_branch.last_revision():
-                source_branch = Branch.open(source_branch_url)
+                target_branch.set_parent(source_branch.base)
+            if source_branch.last_revision() != target_branch.last_revision():
                 # Check if target_branch contains a subset of 
                 # source_branch. If that is not the case, 
                 # assume that source_branch has been replaced 
