@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -29,13 +29,14 @@ from svn.core import Pool
 import svn.core
 
 from bzrlib.plugins.svn.errors import InvalidFileName
-from mapping import (SVN_PROP_BZR_ANCESTRY, SVN_PROP_BZR_MERGE, 
+from logwalker import lazy_dict
+from bzrlib.plugins.svn.mapping import (SVN_PROP_BZR_MERGE, 
                      SVN_PROP_BZR_PREFIX, SVN_PROP_BZR_REVISION_INFO, 
-                     SVN_PROP_BZR_BRANCHING_SCHEME, SVN_PROP_BZR_REVISION_ID,
+                     SVN_PROP_BZR_REVISION_ID,
                      SVN_PROP_BZR_FILEIDS, SVN_REVPROP_BZR_SIGNATURE,
                      parse_merge_property,
                      parse_revision_metadata)
-from repository import (SvnRepository, SvnRepositoryFormat)
+from repository import SvnRepository, SvnRepositoryFormat
 from svk import SVN_PROP_SVK_MERGE
 from tree import (apply_txdelta_handler, parse_externals_description, 
                   inventory_add_external)
@@ -93,29 +94,29 @@ class RevisionBuildEditor(svn.delta.Editor):
         self.source = source
         self.transact = target.get_transaction()
 
-    def start_revision(self, revid, prev_inventory):
+    def start_revision(self, revid, prev_inventory, revmeta):
         self.revid = revid
         (self.branch_path, self.revnum, self.mapping) = self.source.lookup_revision_id(revid)
-        self.svn_revprops = self.source._log._get_transport().revprop_list(self.revnum)
-        changes = self.source._log.get_revision_paths(self.revnum, self.branch_path)
-        renames = self.source.revision_fileid_renames(self.branch_path, self.revnum, self.mapping, 
-                                                      revprops=self.svn_revprops)
-        self.id_map = self.source.transform_fileid_map(self.source.uuid, 
-                              self.revnum, self.branch_path, changes, renames, 
-                              self.mapping)
+        self.revmeta = revmeta
+        self._id_map = None
         self.dir_baserev = {}
         self._revinfo = None
-        self._bzr_merges = ()
-        self._svk_merges = []
         self._premature_deletes = set()
         self.pool = Pool()
         self.old_inventory = prev_inventory
         self.inventory = prev_inventory.copy()
-        self._branch_fileprops = {}
         self._start_revision()
 
-    def _get_parent_ids(self):
-        return self.source.revision_parents(self.revid, self._branch_fileprops)
+    def _get_id_map(self):
+        if self._id_map is not None:
+            return self._id_map
+
+        renames = self.mapping.import_fileid_map(self.revmeta.revprops, self.revmeta.fileprops)
+        self._id_map = self.source.transform_fileid_map(self.source.uuid, 
+                              self.revnum, self.branch_path, self.revmeta.paths, renames, 
+                              self.mapping)
+
+        return self._id_map
 
     def _get_revision(self, revid):
         """Creates the revision object.
@@ -124,11 +125,11 @@ class RevisionBuildEditor(svn.delta.Editor):
         """
 
         # Commit SVN revision properties to a Revision object
-        rev = Revision(revision_id=revid, parent_ids=self._get_parent_ids())
+        rev = Revision(revision_id=revid, parent_ids=self.revmeta.get_parent_ids(self.mapping))
 
-        self.mapping.import_revision(self.svn_revprops, self._branch_fileprops, rev)
+        self.mapping.import_revision(self.revmeta.revprops, self.revmeta.fileprops, rev)
 
-        signature = self.svn_revprops.get(SVN_REVPROP_BZR_SIGNATURE)
+        signature = self.revmeta.revprops.get(SVN_REVPROP_BZR_SIGNATURE)
 
         return (rev, signature)
 
@@ -141,10 +142,7 @@ class RevisionBuildEditor(svn.delta.Editor):
         else:
             assert self.old_inventory.root.revision is not None
             old_file_id = self.old_inventory.root.file_id
-            if self.id_map.has_key(""):
-                file_id = self.id_map[""]
-            else:
-                file_id = old_file_id
+            file_id = self._get_id_map().get("", old_file_id)
             self.dir_baserev[file_id] = [self.old_inventory.root.revision]
 
         if self.inventory.root is not None and \
@@ -159,8 +157,9 @@ class RevisionBuildEditor(svn.delta.Editor):
         assert isinstance(path, unicode)
         assert isinstance(old_parent_id, str)
         assert isinstance(new_parent_id, str)
-        if self.id_map.has_key(path):
-            return self.id_map[path]
+        ret = self._get_id_map().get(path)
+        if ret is not None:
+            return ret
         return self.old_inventory[old_parent_id].children[urlutils.basename(path)].file_id
 
     def _get_old_id(self, parent_id, old_path):
@@ -171,8 +170,9 @@ class RevisionBuildEditor(svn.delta.Editor):
     def _get_new_id(self, parent_id, new_path):
         assert isinstance(new_path, unicode)
         assert isinstance(parent_id, str)
-        if self.id_map.has_key(new_path):
-            return self.id_map[new_path]
+        ret = self._get_id_map().get(new_path)
+        if ret is not None:
+            return ret
         return self.mapping.generate_file_id(self.source.uuid, self.revnum, self.branch_path, new_path)
 
     def _rename(self, file_id, parent_id, path):
@@ -256,39 +256,21 @@ class RevisionBuildEditor(svn.delta.Editor):
 
     def change_dir_prop(self, (old_id, new_id), name, value, pool):
         if new_id == self.inventory.root.file_id:
-            self._branch_fileprops[name] = value
+            # Replay lazy_dict, since it may be more expensive
+            if type(self.revmeta.fileprops) != dict:
+                self.revmeta.fileprops = {}
+            self.revmeta.fileprops[name] = value
 
-        if name == SVN_PROP_BZR_BRANCHING_SCHEME:
-            if new_id != self.inventory.root.file_id:
-                mutter('rogue %r on non-root directory' % name)
-                return
-        elif name == SVN_PROP_BZR_ANCESTRY+str(self.mapping.scheme):
-            if new_id != self.inventory.root.file_id:
-                mutter('rogue %r on non-root directory' % name)
-                return
-            
-            self._bzr_merges = parse_merge_property(value.splitlines()[-1])
-        elif name == SVN_PROP_SVK_MERGE:
-            self._svk_merges = None # Force Repository.revision_parents() to look it up
-        elif name == SVN_PROP_BZR_REVISION_INFO:
-            if new_id != self.inventory.root.file_id:
-                mutter('rogue %r on non-root directory' % SVN_PROP_BZR_REVISION_INFO)
-                return
- 
-        elif name in (svn.core.SVN_PROP_ENTRY_COMMITTED_DATE,
+        if name in (svn.core.SVN_PROP_ENTRY_COMMITTED_DATE,
                       svn.core.SVN_PROP_ENTRY_COMMITTED_REV,
                       svn.core.SVN_PROP_ENTRY_LAST_AUTHOR,
                       svn.core.SVN_PROP_ENTRY_LOCK_TOKEN,
                       svn.core.SVN_PROP_ENTRY_UUID,
-                      svn.core.SVN_PROP_EXECUTABLE,
-                      SVN_PROP_BZR_MERGE, SVN_PROP_BZR_FILEIDS):
+                      svn.core.SVN_PROP_EXECUTABLE):
             pass
-        elif (name.startswith(SVN_PROP_BZR_ANCESTRY) or 
-              name.startswith(SVN_PROP_BZR_REVISION_ID) or 
-              name.startswith(svn.core.SVN_PROP_WC_PREFIX)):
+        elif (name.startswith(svn.core.SVN_PROP_WC_PREFIX)):
             pass
-        elif (name.startswith(svn.core.SVN_PROP_PREFIX) or
-              name.startswith(SVN_PROP_BZR_PREFIX)):
+        elif name.startswith(svn.core.SVN_PROP_PREFIX):
             mutter('unsupported dir property %r' % name)
 
     def change_file_prop(self, id, name, value, pool):
@@ -458,8 +440,7 @@ class WeaveRevisionBuildEditor(RevisionBuildEditor):
         self.inventory.revision_id = self.revid
         # Escaping the commit message is really the task of the serialiser
         rev.message = _escape_commit_message(rev.message)
-        rev.inventory_sha1 = osutils.sha_string(
-                self.target.serialise_inventory(self.inventory))
+        rev.inventory_sha1 = None
         self.target.add_revision(self.revid, rev, self.inventory)
         if signature is not None:
             self.target.add_signature_text(self.revid, signature)
@@ -520,63 +501,82 @@ class InterFromSvnRepository(InterRepository):
     def _get_repo_format_to_test():
         return None
 
-    def _find_all(self):
+    def _find_all(self, mapping, pb=None):
         """Find all revisions from the source repository that are not 
         yet in the target repository.
         """
         parents = {}
-        needed = filter(lambda x: not self.target.has_revision(x), 
-                        self.source.all_revision_ids())
-        for revid in needed:
-            (branch, revnum, mapping) = self.source.lookup_revision_id(revid)
-            parents[revid] = self.source.lhs_revision_parent(branch, revnum, mapping)
-        needed.reverse()
-        return (needed, parents)
+        meta_map = {}
+        graph = self.source.get_graph()
+        available_revs = set()
+        for revmeta in self.source.iter_all_changes(pb=pb):
+            revid = revmeta.get_revision_id(mapping)
+            available_revs.add(revid)
+            meta_map[revid] = revmeta
+        missing = available_revs.difference(self.target.has_revisions(available_revs))
+        needed = list(graph.iter_topo_order(missing))
+        parents = graph.get_parent_map(needed)
+        return [(revid, parents[revid][0], meta_map[revid]) for revid in needed]
 
-    def _find_branches(self, branches, find_ghosts=False):
+    def _find_branches(self, branches, find_ghosts=False, fetch_rhs_ancestry=False, pb=None):
         set_needed = set()
         ret_needed = list()
-        ret_parents = dict()
         for revid in branches:
-            (needed, parents) = self._find_until(revid, find_ghosts=find_ghosts)
-            for rev in needed:
-                if not rev in set_needed:
-                    ret_needed.append(rev)
-                    set_needed.add(rev)
-            ret_parents.update(parents)
-        return ret_needed, ret_parents
+            if pb:
+                pb.update("determining revisions to fetch", branches.index(revid), len(branches))
+            try:
+                nestedpb = ui.ui_factory.nested_progress_bar()
+                for rev in self._find_until(revid, find_ghosts=find_ghosts, fetch_rhs_ancestry=False,
+                                            pb=nestedpb):
+                    if rev[0] not in set_needed:
+                        ret_needed.append(rev)
+                        set_needed.add(rev[0])
+            finally:
+                nestedpb.finished()
+        return ret_needed
 
-    def _find_until(self, revision_id, find_ghosts=False):
+    def _find_until(self, revision_id, find_ghosts=False, fetch_rhs_ancestry=False, pb=None):
         """Find all missing revisions until revision_id
 
         :param revision_id: Stop revision
         :param find_ghosts: Find ghosts
+        :param fetch_rhs_ancestry: Fetch right hand side ancestors
         :return: Tuple with revisions missing and a dictionary with 
             parents for those revision.
         """
+        extra = set()
         needed = []
-        parents = {}
-
-        prev_revid = None
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            for revid in self.source.iter_lhs_ancestry(revision_id, pb):
-
-                if prev_revid is not None:
-                    parents[prev_revid] = revid
-
-                prev_revid = revid
-
+        revs = []
+        meta_map = {}
+        lhs_parent = {}
+        def check_revid(revision_id):
+            prev = None
+            (branch_path, revnum, mapping) = self.source.lookup_revision_id(revision_id)
+            for revmeta in self.source.iter_reverse_branch_changes(branch_path, revnum, mapping):
+                if pb:
+                    pb.update("determining revisions to fetch", revnum-revmeta.revnum, revnum)
+                revid = revmeta.get_revision_id(mapping)
+                lhs_parent[prev] = revid
+                meta_map[revid] = revmeta
+                if fetch_rhs_ancestry:
+                    extra.update(revmeta.get_rhs_parents(mapping))
                 if not self.target.has_revision(revid):
-                    needed.append(revid)
+                    revs.append(revid)
                 elif not find_ghosts:
+                    prev = None
                     break
-        finally:
-            pb.finished()
+                prev = revid
+            lhs_parent[prev] = NULL_REVISION
 
-        parents[prev_revid] = None
-        needed.reverse()
-        return (needed, parents)
+        check_revid(revision_id)
+
+        for revid in extra:
+            if revid not in revs:
+                check_revid(revid)
+
+        needed = [(revid, lhs_parent[revid], meta_map[revid]) for revid in reversed(revs)]
+
+        return needed
 
     def copy_content(self, revision_id=None, pb=None):
         """See InterRepository.copy_content."""
@@ -590,17 +590,16 @@ class InterFromSvnRepository(InterRepository):
         """
         raise NotImplementedError(self._copy_revisions_replay)
 
-    def _fetch_switch(self, revids, pb=None, lhs_parent=None):
+    def _fetch_switch(self, conn, revids, pb=None):
         """Copy a set of related revisions using svn.ra.switch.
 
         :param revids: List of revision ids of revisions to copy, 
                        newest first.
         :param pb: Optional progress bar.
         """
-        repos_root = self.source.transport.get_svn_repos_root()
+        repos_root = conn.get_repos_root()
 
         prev_revid = None
-        transport = self.source.transport
         if pb is None:
             pb = ui.ui_factory.nested_progress_bar()
             nested_pb = pb
@@ -614,30 +613,30 @@ class InterFromSvnRepository(InterRepository):
         editor = revbuildklass(self.source, self.target)
 
         try:
-            for revid in revids:
+            for (revid, parent_revid, revmeta) in revids:
                 pb.update('copying revision', num, len(revids))
 
-                parent_revid = lhs_parent[revid]
+                assert parent_revid is not None
 
-                if parent_revid is None:
+                if parent_revid == NULL_REVISION:
                     parent_inv = Inventory(root_id=None)
                 elif prev_revid != parent_revid:
                     parent_inv = self.target.get_inventory(parent_revid)
                 else:
                     parent_inv = prev_inv
 
-                editor.start_revision(revid, parent_inv)
+                editor.start_revision(revid, parent_inv, revmeta)
 
                 try:
                     pool = Pool()
 
-                    if parent_revid is None:
+                    if parent_revid == NULL_REVISION:
                         branch_url = urlutils.join(repos_root, 
                                                    editor.branch_path)
-                        transport.reparent(branch_url)
-                        assert transport.svn_url == branch_url.rstrip("/"), \
-                            "Expected %r, got %r" % (transport.svn_url, branch_url)
-                        reporter = transport.do_update(editor.revnum, True, 
+                        conn.reparent(branch_url)
+                        assert conn.url == branch_url, \
+                            "Expected %r, got %r" % (conn.url, branch_url)
+                        reporter = conn.do_update(editor.revnum, True, 
                                                        editor, pool)
 
                         # Report status of existing paths
@@ -645,21 +644,19 @@ class InterFromSvnRepository(InterRepository):
                     else:
                         (parent_branch, parent_revnum, mapping) = \
                                 self.source.lookup_revision_id(parent_revid)
-                        transport.reparent(urlutils.join(repos_root, parent_branch))
+                        conn.reparent(urlutils.join(repos_root, parent_branch))
 
                         if parent_branch != editor.branch_path:
-                            reporter = transport.do_switch(editor.revnum, True, 
+                            reporter = conn.do_switch(editor.revnum, True, 
                                 urlutils.join(repos_root, editor.branch_path), 
                                 editor, pool)
                         else:
-                            reporter = transport.do_update(editor.revnum, True, editor)
+                            reporter = conn.do_update(editor.revnum, True, editor)
 
                         # Report status of existing paths
                         reporter.set_path("", parent_revnum, False, None, pool)
 
-                    lock = transport.lock_read(".")
                     reporter.finish_report(pool)
-                    lock.unlock()
                 except:
                     editor.abort_edit()
                     raise
@@ -672,28 +669,28 @@ class InterFromSvnRepository(InterRepository):
             self.target.unlock()
             if nested_pb is not None:
                 nested_pb.finished()
-        self.source.transport.reparent_root()
 
     def fetch(self, revision_id=None, pb=None, find_ghosts=False, 
-              branches=None):
+              branches=None, fetch_rhs_ancestry=False):
         """Fetch revisions. """
         if revision_id == NULL_REVISION:
             return
         # Dictionary with paths as keys, revnums as values
 
+        if pb:
+            pb.update("determining revisions to fetch", 0, 2)
+
         # Loop over all the revnums until revision_id
         # (or youngest_revnum) and call self.target.add_revision() 
         # or self.target.add_inventory() each time
         self.target.lock_read()
         try:
             if branches is not None:
-                (needed, lhs_parent) = self._find_branches(branches, 
-                                                           find_ghosts)
+                needed = self._find_branches(branches, find_ghosts, fetch_rhs_ancestry, pb=pb)
             elif revision_id is None:
-                (needed, lhs_parent) = self._find_all()
+                needed = self._find_all(self.source.get_mapping(), pb=pb)
             else:
-                (needed, lhs_parent) = self._find_until(revision_id, 
-                                                        find_ghosts)
+                needed = self._find_until(revision_id, find_ghosts, fetch_rhs_ancestry, pb=pb)
         finally:
             self.target.unlock()
 
@@ -701,7 +698,11 @@ class InterFromSvnRepository(InterRepository):
             # Nothing to fetch
             return
 
-        self._fetch_switch(needed, pb, lhs_parent)
+        conn = self.source.transport.get_connection()
+        try:
+            self._fetch_switch(conn, needed, pb)
+        finally:
+            self.source.transport.add_connection(conn)
 
     @staticmethod
     def is_compatible(source, target):
