@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -27,7 +27,7 @@ import svn.core, svn.repos
 from errors import NoSvnRepositoryPresent
 from format import get_rich_root_format, SvnRemoteFormat
 from repository import SvnRepository
-from transport import SvnRaTransport, bzr_to_svn_url, get_svn_ra_transport
+from transport import bzr_to_svn_url, get_svn_ra_transport
 
 
 class SvnRemoteAccess(BzrDir):
@@ -140,14 +140,22 @@ class SvnRemoteAccess(BzrDir):
             stop_revision = source.last_revision()
         target_branch_path = self.branch_path.strip("/")
         repos = self.find_repository()
-        full_branch_url = urlutils.join(repos.transport.base, 
-                                        target_branch_path)
-        if repos.transport.check_path(target_branch_path,
-            repos.transport.get_latest_revnum()) != svn.core.svn_node_none:
-            raise AlreadyBranchError(full_branch_url)
-        push_new(repos, target_branch_path, source, stop_revision)
+        repos.lock_write()
+        try:
+            full_branch_url = urlutils.join(repos.transport.base, 
+                                            target_branch_path)
+            if repos.transport.check_path(target_branch_path,
+                repos.get_latest_revnum()) != svn.core.svn_node_none:
+                raise AlreadyBranchError(full_branch_url)
+            push_new(repos, target_branch_path, source, stop_revision)
+        finally:
+            repos.unlock()
         branch = self.open_branch()
-        branch.pull(source, stop_revision=stop_revision)
+        branch.lock_write()
+        try:
+            branch.pull(source, stop_revision=stop_revision)
+        finally:
+            branch.unlock()
         return branch
 
     def create_branch(self):
@@ -158,7 +166,7 @@ class SvnRemoteAccess(BzrDir):
         if self.branch_path != "":
             # TODO: Set NULL_REVISION in SVN_PROP_BZR_BRANCHING_SCHEME
             repos.transport.mkdir(self.branch_path.strip("/"))
-        elif repos.transport.get_latest_revnum() > 0:
+        elif repos.get_latest_revnum() > 0:
             # Bail out if there are already revisions in this repository
             raise AlreadyBranchError(self.root_transport.base)
         branch = SvnBranch(self.root_transport.base, repos, self.branch_path)
