@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -19,8 +19,8 @@ from bzrlib.repository import Repository
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.trace import mutter
 
-from mapping import BzrSvnMappingv3FileProps
-from scheme import NoBranchingScheme
+from mapping3 import BzrSvnMappingv3FileProps
+from mapping3.scheme import NoBranchingScheme
 from tests import TestCaseWithSubversionRepository
 
 import os
