@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -20,6 +20,7 @@ from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NoSuchFile, NoSuchRevision, NotBranchError
 from bzrlib.repository import Repository
+from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 
 import os
@@ -27,8 +28,9 @@ from unittest import TestCase
 
 from branch import FakeControlFiles, SvnBranchFormat
 from convert import load_dumpfile
-from mapping import (SVN_PROP_BZR_REVISION_ID, BzrSvnMappingv3FileProps)
-from scheme import TrunkBranchingScheme
+from mapping import SVN_PROP_BZR_REVISION_ID
+from mapping3 import BzrSvnMappingv3FileProps
+from mapping3.scheme import TrunkBranchingScheme
 from tests import TestCaseWithSubversionRepository
 
 class WorkingSubversionBranch(TestCaseWithSubversionRepository):
@@ -192,7 +194,7 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         """The None revid should map to revno 0."""
         repos_url = self.make_client('a', 'dc')
         branch = Branch.open(repos_url)
-        self.assertEquals(0, branch.revision_id_to_revno(None))
+        self.assertEquals(0, branch.revision_id_to_revno(NULL_REVISION))
 
     def test_revision_id_to_revno_nonexistant(self):
         """revision_id_to_revno() should raise NoSuchRevision if
@@ -502,6 +504,7 @@ foohosts""")
         weave = newbranch.repository.weave_store.get_weave(
             tree.inventory.path2id("hosts"),
             newbranch.repository.get_transaction())
+
         self.assertEqual(set([
             oldbranch.generate_revision_id(6),
             oldbranch.generate_revision_id(7)]),
@@ -711,6 +714,7 @@ foohosts""")
         newdir.find_repository().get_revision_inventory(
                 newdir.open_branch().last_revision())
 
+
 class TestFakeControlFiles(TestCase):
     def test_get_utf8(self):
         f = FakeControlFiles()
@@ -721,6 +725,7 @@ class TestFakeControlFiles(TestCase):
         f = FakeControlFiles()
         self.assertRaises(NoSuchFile, f.get, "foobla")
 
+
 class BranchFormatTests(TestCase):
     def setUp(self):
         self.format = SvnBranchFormat()
