@@ -4,7 +4,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -29,7 +29,7 @@ from commit import set_svn_revprops, _revision_id_to_svk_feature
 from copy import copy
 from errors import RevpropChangeFailed
 import os
-from remote import SvnRaTransport
+from transport import SvnRaTransport
 from tests import TestCaseWithSubversionRepository
 
 from svn.core import svn_time_to_cstring
@@ -132,8 +132,8 @@ class TestNativeCommit(TestCaseWithSubversionRepository):
         self.assertEqual("some-ghost-revision\n", 
                 self.client_get_prop(repos_url, "bzr:ancestry:v3-none", 1))
         self.assertEqual((wt.branch.generate_revision_id(0), "some-ghost-revision"),
-                         wt.branch.repository.revision_parents(
-                             wt.branch.last_revision()))
+                         wt.branch.repository.get_revision(
+                             wt.branch.last_revision()).parent_ids)
 
     def test_commit_rename_file(self):
         repos_url = self.make_client('d', 'dc')
@@ -535,6 +535,7 @@ class HeavyWeightCheckoutTests(TestCaseWithSubversionRepository):
         tree1 = master_branch.repository.revision_tree(revid1)
         tree2 = master_branch.repository.revision_tree(revid2)
         delta = tree2.changes_from(tree1)
+        mutter("changes %r" % list(master_branch.repository.iter_reverse_branch_changes("", 2, master_branch.repository.get_mapping())))
         self.assertEquals(0, len(delta.added))
         self.assertEquals(0, len(delta.removed))
         self.assertEquals(1, len(delta.renamed))
@@ -582,6 +583,7 @@ class RevpropTests(TestCaseWithSubversionRepository):
         self.assertRaises(RevpropChangeFailed, 
                 lambda: set_svn_revprops(transport, 1, {"svn:author": "Somebody", "svn:date": svn_time_to_cstring(1000000*473385600)}))
 
+
 class SvkTestCase(TestCase):
     def test_revid_svk_map(self):
         self.assertEqual("auuid:/:6", 
