@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -16,8 +16,10 @@
 
 """Config tests."""
 
-from config import SvnRepositoryConfig
-from scheme import TrunkBranchingScheme
+from bzrlib.branch import Branch
+from config import SvnRepositoryConfig, BranchConfig
+from mapping3.scheme import TrunkBranchingScheme
+from tests import TestCaseWithSubversionRepository
 
 from bzrlib.tests import TestCaseInTempDir
 
@@ -64,9 +66,21 @@ class ReposConfigTests(TestCaseInTempDir):
         c = SvnRepositoryConfig("blabla2")
         self.assertEquals(None, c.get_override_svn_revprops())
         c.set_user_option("override-svn-revprops", "True")
-        self.assertEquals(True, c.get_override_svn_revprops())
+        self.assertEquals(["svn:date", "svn:author"], c.get_override_svn_revprops())
         c.set_user_option("override-svn-revprops", "False")
-        self.assertEquals(False, c.get_override_svn_revprops())
+        self.assertEquals([], c.get_override_svn_revprops())
+        c.set_user_option("override-svn-revprops", ["svn:author", "svn:date"])
+        self.assertEquals(["svn:author","svn:date"], c.get_override_svn_revprops())
+        c.set_user_option("override-svn-revprops", ["svn:author"])
+        self.assertEquals(["svn:author"], c.get_override_svn_revprops())
+
+    def test_get_append_revisions_only(self):
+        c = SvnRepositoryConfig("blabla2")
+        self.assertEquals(None, c.get_append_revisions_only())
+        c.set_user_option("append_revisions_only", "True")
+        self.assertEquals(True, c.get_append_revisions_only())
+        c.set_user_option("append_revisions_only", "False")
+        self.assertEquals(False, c.get_append_revisions_only())
 
     def test_set_revprops(self):
         c = SvnRepositoryConfig("blabla2")
@@ -76,6 +90,14 @@ class ReposConfigTests(TestCaseInTempDir):
         c.set_user_option("set-revprops", "False")
         self.assertEquals(False, c.get_set_revprops())
 
+    def test_log_strip_trailing_newline(self):
+        c = SvnRepositoryConfig("blabla3")
+        self.assertEquals(False, c.get_log_strip_trailing_newline())
+        c.set_user_option("log-strip-trailing-newline", "True")
+        self.assertEquals(True, c.get_log_strip_trailing_newline())
+        c.set_user_option("log-strip-trailing-newline", "False")
+        self.assertEquals(False, c.get_log_strip_trailing_newline())
+
     def test_supports_change_revprop(self):
         c = SvnRepositoryConfig("blabla2")
         self.assertEquals(None, c.get_supports_change_revprop())
@@ -83,3 +105,16 @@ class ReposConfigTests(TestCaseInTempDir):
         self.assertEquals(True, c.get_supports_change_revprop())
         c.set_user_option("supports-change-revprop", "False")
         self.assertEquals(False, c.get_supports_change_revprop())
+
+
+class BranchConfigTests(TestCaseWithSubversionRepository):
+    def setUp(self):
+        super(BranchConfigTests, self).setUp()
+        self.repos_url = self.make_client("d", "dc")
+        self.config = BranchConfig(Branch.open(self.repos_url))
+
+    def test_set_option(self):
+        self.config.set_user_option("append_revisions_only", "True")
+        self.assertEquals("True", self.config.get_user_option("append_revisions_only"))
+
+
