@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -21,12 +21,12 @@ from bzrlib.repository import Repository
 from bzrlib.trace import mutter
 from bzrlib.tests import TestCase
 
-from fileids import SimpleFileIdMap, simple_apply_changes
-from mapping import BzrSvnMappingv3FileProps
-from scheme import TrunkBranchingScheme, NoBranchingScheme
+from fileids import simple_apply_changes
+from mapping3 import BzrSvnMappingv3FileProps, set_branching_scheme
+from mapping3.scheme import TrunkBranchingScheme, NoBranchingScheme
 from tests import TestCaseWithSubversionRepository
 
-class MockRepo:
+class MockRepo(object):
     def __init__(self, mapping, uuid="uuid"):
         self.uuid = uuid
 
@@ -161,6 +161,7 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         self.assertEqual(fileid, inv1.path2id("dir/file"))
         self.assertEqual(repository.generate_revision_id(1, "trunk", mapping), revid)
 
+
 class TestFileMapping(TestCase):
     def setUp(self):
         self.mapping = BzrSvnMappingv3FileProps(NoBranchingScheme())
@@ -245,6 +246,7 @@ class TestFileMapping(TestCase):
         self.assertEqual("1@uuid::foo", map["foo"][0])
         self.assertEqual((1, ""), map["foo"][1])
 
+
 class GetMapTests(TestCaseWithSubversionRepository):
     def setUp(self):
         super(GetMapTests, self).setUp()
@@ -252,13 +254,13 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.repos = Repository.open(self.repos_url)
 
     def test_empty(self):
-        self.repos.set_branching_scheme(NoBranchingScheme())
+        set_branching_scheme(self.repos, NoBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.assertEqual({"": (self.mapping.generate_file_id(self.repos.uuid, 0, "", u""), self.repos.generate_revision_id(0, "", self.mapping))}, 
                          self.repos.get_fileid_map(0, "", self.mapping))
 
     def test_empty_trunk(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
@@ -267,7 +269,7 @@ class GetMapTests(TestCaseWithSubversionRepository):
                 self.repos.get_fileid_map(1, "trunk", self.mapping))
 
     def test_change_parent(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
@@ -278,7 +280,7 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.assertEqual({"": (self.mapping.generate_file_id(self.repos.uuid, 1, "trunk", u""), self.repos.generate_revision_id(2, "trunk", self.mapping)), "file": (self.mapping.generate_file_id(self.repos.uuid, 2, "trunk", u"file"), self.repos.generate_revision_id(2, "trunk", self.mapping))}, self.repos.get_fileid_map(2, "trunk", self.mapping))
 
     def test_change_updates(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
@@ -291,7 +293,7 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.assertEqual({"": (self.mapping.generate_file_id(self.repos.uuid, 1, "trunk", u""), self.repos.generate_revision_id(3, "trunk", self.mapping)), "file": (self.mapping.generate_file_id(self.repos.uuid, 2, "trunk", u"file"), self.repos.generate_revision_id(3, "trunk", self.mapping))}, self.repos.get_fileid_map(3, "trunk", self.mapping))
 
     def test_sibling_unrelated(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
@@ -308,7 +310,7 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.assertEqual({"": (self.mapping.generate_file_id(self.repos.uuid, 1, "trunk", u""), self.repos.generate_revision_id(3, "trunk", self.mapping)), "bar": (self.mapping.generate_file_id(self.repos.uuid, 2, "trunk", u"bar"), self.repos.generate_revision_id(2, "trunk", self.mapping)), "file": (self.mapping.generate_file_id(self.repos.uuid, 2, "trunk", u"file"), self.repos.generate_revision_id(3, "trunk", self.mapping))}, self.repos.get_fileid_map(3, "trunk", self.mapping))
 
     def test_copy(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
@@ -323,7 +325,7 @@ class GetMapTests(TestCaseWithSubversionRepository):
             "bar": (self.mapping.generate_file_id(self.repos.uuid, 3, "trunk", u"bar"), self.repos.generate_revision_id(3, "trunk", self.mapping)), "file": (self.mapping.generate_file_id(self.repos.uuid, 2, "trunk", u"file"), self.repos.generate_revision_id(2, "trunk", self.mapping))}, self.repos.get_fileid_map(3, "trunk", self.mapping))
 
     def test_copy_nested_modified(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(self.repos, TrunkBranchingScheme())
         self.mapping = self.repos.get_mapping()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
