@@ -21,9 +21,10 @@ from errors import InvalidPropertyValue
 from mapping import (generate_revision_metadata, parse_revision_metadata, 
                      parse_revid_property, parse_merge_property, 
                      BzrSvnMappingv1, BzrSvnMappingv2, 
-                     BzrSvnMappingv3FileProps, BzrSvnMappingv3RevProps,
-                     BzrSvnMappingv4, BzrSvnMappingv3Hybrid, parse_revision_id)
-from scheme import NoBranchingScheme
+                     BzrSvnMappingv4, parse_revision_id)
+from mapping3 import (BzrSvnMappingv3FileProps, BzrSvnMappingv3RevProps, 
+                      BzrSvnMappingv3Hybrid)
+from mapping3.scheme import NoBranchingScheme
 
 from bzrlib.errors import InvalidRevisionId
 from bzrlib.tests import (TestCase, adapt_tests, TestNotApplicable)
@@ -123,7 +124,7 @@ class ParseMergePropertyTestCase(TestCase):
         self.assertEqual(("bla", "bloe"), parse_merge_property("bla\tbloe"))
 
 
-class MappingTestAdapter:
+class MappingTestAdapter(object):
     def test_roundtrip_revision(self):
         revid = self.mapping.generate_revision_id("myuuid", 42, "path")
         (uuid, path, revnum, mapping) = self.mapping.parse_revision_id(revid)
@@ -142,6 +143,19 @@ class MappingTestAdapter:
         self.assertEquals(fileids, 
                 self.mapping.import_fileid_map(revprops, fileprops))
 
+    def test_message(self):
+        if not self.mapping.supports_roundtripping():
+            raise TestNotApplicable
+        (revprops, fileprops) = self.mapping.export_revision("branchp", 432432432.0, 0, "somebody", 
+                                     {"arevprop": "val"}, "arevid", 4, ["merge1"], dict())
+        try:
+            self.mapping.export_message("My Commit message", revprops, fileprops)
+        except NotImplementedError:
+            raise TestNotApplicable
+        targetrev = Revision(None)
+        self.mapping.import_revision(revprops, fileprops, targetrev)
+        self.assertEquals("My Commit message", targetrev.message)
+
     def test_revision(self):
         if not self.mapping.supports_roundtripping():
             raise TestNotApplicable
@@ -268,7 +282,7 @@ class Mappingv4TestAdapter(MappingTestAdapter,TestCase):
         self.mapping = BzrSvnMappingv4()
 
 
-class ParseRevisionIdTests:
+class ParseRevisionIdTests(object):
     def test_current(self):
         self.assertEqual(("uuid", "trunk", 1, BzrSvnMappingv3FileProps(TrunkBranchingScheme())), 
                 parse_revision_id("svn-v3-trunk0:uuid:trunk:1"))
