@@ -4,7 +4,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -31,12 +31,14 @@ from bzrlib.tests import TestCase
 
 import os, sys
 
-import svn.fs
+import svn.fs, svn
 
 import format
 from mapping import (escape_svn_path, unescape_svn_path, 
-                     SVN_PROP_BZR_REVISION_ID, SVN_PROP_BZR_BRANCHING_SCHEME)
-from scheme import (TrunkBranchingScheme, NoBranchingScheme, 
+                     SVN_PROP_BZR_REVISION_ID)
+from mapping3 import (SVN_PROP_BZR_BRANCHING_SCHEME, set_branching_scheme,
+                      set_property_scheme)
+from mapping3.scheme import (TrunkBranchingScheme, NoBranchingScheme, 
                     ListBranchingScheme, SingleBranchingScheme)
 from transport import SvnRaTransport
 from tests import TestCaseWithSubversionRepository
@@ -64,7 +66,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.assertEqual([
             ('', {'foo': ('A', None, -1)}, 1), 
             ('', {'': ('A', None, -1)}, 0)],
-             list(repos.follow_branch_history("", 1, NoBranchingScheme())))
+            [(l.branch_path, l.paths, l.revnum) for l in repos.iter_reverse_branch_changes("", 1, NoBranchingScheme())])
 
     def test_make_working_trees(self):
         repos_url = self.make_client("a", "dc")
@@ -163,7 +165,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
         repos = Repository.open(repos_url)
         self.assertRaises(NoSuchRevision, list, 
-               repos.follow_branch_history("/", 20, NoBranchingScheme()))
+               repos.iter_reverse_branch_changes("/", 20, NoBranchingScheme()))
 
     def test_follow_branch_switched_parents(self):
         repos_url = self.make_client('a', 'dc')
@@ -178,8 +180,17 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_update("dc")
         self.client_commit("dc", "commit")
         repos = Repository.open(repos_url)
-        self.assertEquals([('pygments/trunk', 3), ('pykleur/trunk', 2), ('pykleur/trunk', 1)], 
-                list(repos.follow_branch("pygments/trunk", 3, TrunkBranchingScheme(1))))
+        self.assertEquals([
+            ('pygments/trunk', {'pygments': (u'A', 'pykleur', 1),
+                                'pygments/trunk': (u'R', 'pykleur/trunk', 2),
+                                'pykleur': (u'D', None, -1)}, 3),
+            ('pykleur/trunk', {'pykleur/trunk/pykleur/afile': (u'A', None, -1)}, 2),
+            ('pykleur/trunk',
+                    {'pykleur': (u'A', None, -1),
+                     'pykleur/trunk': (u'A', None, -1),
+                     'pykleur/trunk/pykleur': (u'A', None, -1)},
+             1)],
+            [(l.branch_path, l.paths, l.revnum) for l in repos.iter_reverse_branch_changes("pygments/trunk", 3, TrunkBranchingScheme(1))])
 
     def test_follow_branch_move_single(self):
         repos_url = self.make_client('a', 'dc')
@@ -190,8 +201,11 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_update("dc")
         self.client_commit("dc", "commit")
         repos = Repository.open(repos_url)
-        self.assertEquals([('pygments', 2)], 
-                list(repos.follow_branch("pygments", 2, SingleBranchingScheme("pygments"))))
+        changes = repos.iter_reverse_branch_changes("pygments", 2, SingleBranchingScheme("pygments"))
+        self.assertEquals([('pygments',
+              {'pygments/bla': ('A', None, -1), 'pygments': ('A', None, -1)},
+                2)],
+                [(l.branch_path, l.paths, l.revnum) for l in changes])
 
     def test_history_all(self):
         repos_url = self.make_client("a", "dc")
@@ -203,12 +217,12 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos = Repository.open(repos_url)
 
         self.assertEqual(2, 
-                   len(list(repos.follow_history(1, repos.get_mapping()))))
+                   len(list(repos.all_revision_ids(repos.get_layout()))))
 
     def test_all_revs_empty(self):
         repos_url = self.make_client("a", "dc")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
         self.assertEqual([], list(repos.all_revision_ids()))
 
     def test_all_revs(self):
@@ -223,19 +237,18 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "remove branch")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
         mapping = repos.get_mapping()
         self.assertEqual([
-            repos.generate_revision_id(2, "branches/somebranch", mapping),
-            repos.generate_revision_id(1, "trunk", mapping)], 
+            repos.generate_revision_id(1, "trunk", mapping), 
+            repos.generate_revision_id(2, "branches/somebranch", mapping)],
             list(repos.all_revision_ids()))
 
     def test_follow_history_empty(self):
         repos_url = self.make_client("a", "dc")
         repos = Repository.open(repos_url)
-        self.assertEqual([('', 0)], 
-              list(repos.follow_history(0, 
-                  repos.get_mapping())))
+        self.assertEqual([repos.generate_revision_id(0, '', repos.get_mapping())], 
+              list(repos.all_revision_ids(repos.get_layout())))
 
     def test_follow_history_empty_branch(self):
         repos_url = self.make_client("a", "dc")
@@ -244,9 +257,9 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_add("dc/branches")
         self.client_commit("dc", "My Message")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
-        self.assertEqual([('trunk', 1)], 
-                list(repos.follow_history(1, repos.get_mapping())))
+        set_branching_scheme(repos, TrunkBranchingScheme())
+        self.assertEqual([repos.generate_revision_id(1, 'trunk', repos.get_mapping())], 
+                list(repos.all_revision_ids(repos.get_layout())))
 
     def test_follow_history_follow(self):
         repos_url = self.make_client("a", "dc")
@@ -259,11 +272,12 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "Create branch")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
-        items = list(repos.follow_history(2, repos.get_mapping()))
-        self.assertEqual([('branches/abranch', 2), 
-                          ('trunk', 1)], items)
+        items = list(repos.all_revision_ids(repos.get_layout()))
+        self.assertEqual([repos.generate_revision_id(1, 'trunk', repos.get_mapping()),
+                          repos.generate_revision_id(2, 'branches/abranch', repos.get_mapping())
+                          ], items)
 
     def test_branch_log_specific(self):
         repos_url = self.make_client("a", "dc")
@@ -277,9 +291,9 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
-        self.assertEqual(1, len(list(repos.follow_branch_history("branches/brancha",
+        self.assertEqual(1, len(list(repos.iter_reverse_branch_changes("branches/brancha",
             1, TrunkBranchingScheme()))))
 
     def test_branch_log_specific_ignore(self):
@@ -298,9 +312,9 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message2")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
-        self.assertEqual(1, len(list(repos.follow_branch_history("branches/brancha",
+        self.assertEqual(1, len(list(repos.iter_reverse_branch_changes("branches/brancha",
             2, TrunkBranchingScheme()))))
 
     def test_find_branches(self):
@@ -313,13 +327,13 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_add("dc/branches")
         self.client_commit("dc", "My Message")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
         branches = repos.find_branches()
         self.assertEquals(2, len(branches))
         self.assertEquals(urlutils.join(repos.base, "branches/brancha"), 
-                          branches[0].base)
-        self.assertEquals(urlutils.join(repos.base, "branches/branchab"), 
                           branches[1].base)
+        self.assertEquals(urlutils.join(repos.base, "branches/branchab"), 
+                          branches[0].base)
 
     def test_find_branchpaths_moved(self):
         repos_url = self.make_client("a", "dc")
@@ -334,7 +348,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message 2")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
         self.assertEqual([("tags/branchab", 2, True), 
                           ("tags/brancha", 2, True)], 
@@ -350,7 +364,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message 2")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
         self.assertEqual([("branches/branchb", 2, True)],
                 list(repos.find_branchpaths(TrunkBranchingScheme(), from_revnum=2, 
@@ -398,7 +412,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message 2")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme(1))
+        set_branching_scheme(repos, TrunkBranchingScheme(1))
 
         self.assertEqual([("t2/branches/brancha", 2, True), 
                           ("t2/branches/branchab", 2, True)], 
@@ -408,7 +422,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(NoBranchingScheme())
+        set_branching_scheme(repos, NoBranchingScheme())
 
         self.assertEqual([("", 0, True)], 
                 list(repos.find_branchpaths(NoBranchingScheme(), to_revnum=0)))
@@ -417,7 +431,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(NoBranchingScheme())
+        set_branching_scheme(repos, NoBranchingScheme())
 
         self.assertEqual([("", 0, True)], 
                 list(repos.find_branchpaths(NoBranchingScheme(), to_revnum=0)))
@@ -426,7 +440,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
         self.assertEqual([], 
                 list(repos.find_branchpaths(TrunkBranchingScheme(), to_revnum=0)))
@@ -435,7 +449,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
         self.build_tree({'dc/trunk/foo': "data"})
         self.client_add("dc/trunk")
@@ -448,7 +462,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repos_url = self.make_client("a", "dc")
 
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
 
         self.build_tree({'dc/trunk/foo': "data"})
         self.client_add("dc/trunk")
@@ -545,15 +559,15 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repository = Repository.open("svn+%s" % repos_url)
         mapping = repository.get_mapping()
         self.assertEqual((),
-                repository.revision_parents(
-                    repository.generate_revision_id(0, "", mapping)))
+                repository.get_revision(
+                    repository.generate_revision_id(0, "", mapping)).parent_ids)
         self.assertEqual((repository.generate_revision_id(0, "", mapping),),
-                repository.revision_parents(
-                    repository.generate_revision_id(1, "", mapping)))
+                repository.get_revision(
+                    repository.generate_revision_id(1, "", mapping)).parent_ids)
         self.assertEqual((repository.generate_revision_id(1, "", mapping),
             "ghostparent"), 
-                repository.revision_parents(
-                    repository.generate_revision_id(2, "", mapping)))
+                repository.get_revision(
+                    repository.generate_revision_id(2, "", mapping)).parent_ids)
  
     def test_revision_svk_parent(self):
         repos_url = self.make_client('d', 'dc')
@@ -564,15 +578,15 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_update("dc")
         self.build_tree({'dc/trunk/foo': "data2"})
         repository = Repository.open("svn+%s" % repos_url)
-        repository.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repository, TrunkBranchingScheme())
         self.client_set_prop("dc/trunk", "svk:merge", 
             "%s:/branches/foo:1\n" % repository.uuid)
         self.client_commit("dc", "Second Message")
         mapping = repository.get_mapping()
         self.assertEqual((repository.generate_revision_id(1, "trunk", mapping),
             repository.generate_revision_id(1, "branches/foo", mapping)), 
-                repository.revision_parents(
-                    repository.generate_revision_id(2, "trunk", mapping)))
+                repository.get_revision(
+                    repository.generate_revision_id(2, "trunk", mapping)).parent_ids)
     
     def test_get_revision(self):
         repos_url = self.make_client('d', 'dc')
@@ -634,9 +648,9 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
     def test_store_branching_scheme(self):
         repos_url = self.make_client('d', 'dc')
         repository = Repository.open(repos_url)
-        repository.set_branching_scheme(TrunkBranchingScheme(42))
+        set_branching_scheme(repository, TrunkBranchingScheme(42))
         repository = Repository.open(repos_url)
-        self.assertEquals("trunk42", str(repository.get_scheme()))
+        self.assertEquals("trunk42", str(repository.get_mapping().scheme))
 
     def test_get_ancestry(self):
         repos_url = self.make_client('d', 'dc')
@@ -713,7 +727,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.build_tree({'dc/trunk/a': "bloe"})
         self.client_commit("dc", "second commit")
         repository = Repository.open(repos_url)
-        repository.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repository, TrunkBranchingScheme())
         mapping = repository.get_mapping()
         self.assertEqual({repository.generate_revision_id(1, "trunk", mapping): (),
                           repository.generate_revision_id(2, "trunk", mapping): (repository.generate_revision_id(1, "trunk", mapping),),
@@ -942,12 +956,12 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "set scheme")
         repository = Repository.open("svn+%s" % repos_url)
         self.assertEquals(ListBranchingScheme(["trunk", "branches/*", "branches/tmp/*"]).branch_list,
-                          repository.get_scheme().branch_list)
+                          repository.get_mapping().scheme.branch_list)
 
     def test_set_property_scheme(self):
         repos_url = self.make_client('d', 'dc')
         repos = Repository.open(repos_url)
-        repos.set_property_scheme(ListBranchingScheme(["bla/*"]))
+        set_property_scheme(repos, ListBranchingScheme(["bla/*"]))
         self.client_update("dc")
         self.assertEquals("bla/*\n", 
                    self.client_get_prop("dc", SVN_PROP_BZR_BRANCHING_SCHEME))
@@ -1004,19 +1018,6 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         repository = Repository.open("svn+%s" % repos_url)
         self.assertTrue(repository.is_shared())
 
-    def test_revision_fileid_renames(self):
-        repos_url = self.make_client('d', 'dc')
-        self.build_tree({'dc/test': "data"})
-        self.client_add("dc/test")
-        self.client_set_prop("dc", "bzr:revision-info", "")
-        self.client_set_prop("dc", "bzr:file-ids", "test\tbla\n")
-        self.client_commit("dc", "Msg")
-
-        repos = Repository.open(repos_url)
-        mapping = repos.get_mapping()
-        renames = repos.revision_fileid_renames("", 1, mapping)
-        self.assertEqual({"test": "bla"}, renames)
-
     def test_fetch_property_change_only_trunk(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/trunk/bla': "data"})
@@ -1029,8 +1030,10 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_set_prop("dc/trunk", "some:property", "some other data\n")
         self.client_commit("dc", "My 4")
         oldrepos = Repository.open("svn+"+repos_url)
-        self.assertEquals([('trunk', 3), ('trunk', 2), ('trunk', 1)], 
-            list(oldrepos.follow_branch("trunk", 3, TrunkBranchingScheme())))
+        self.assertEquals([('trunk', {'trunk': (u'M', None, -1)}, 3), 
+                           ('trunk', {'trunk': (u'M', None, -1)}, 2), 
+                           ('trunk', {'trunk/bla': (u'A', None, -1), 'trunk': (u'A', None, -1)}, 1)], 
+                   [(l.branch_path, l.paths, l.revnum) for l in oldrepos.iter_reverse_branch_changes("trunk", 3, TrunkBranchingScheme())])
 
     def test_control_code_msg(self):
         repos_url = self.make_client('d', 'dc')
@@ -1051,7 +1054,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_commit("dc", "foohosts") #4
 
         oldrepos = Repository.open("svn+"+repos_url)
-        oldrepos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(oldrepos, TrunkBranchingScheme())
         dir = BzrDir.create("f",format=format.get_rich_root_format())
         newrepos = dir.create_repository()
         oldrepos.copy_content_into(newrepos)
@@ -1084,18 +1087,18 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
     def test_set_branching_scheme(self):
         repos_url = self.make_client('d', 'dc')
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(NoBranchingScheme())
+        set_branching_scheme(repos, NoBranchingScheme())
 
     def testlhs_revision_parent_none(self):
         repos_url = self.make_client('d', 'dc')
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(NoBranchingScheme())
-        self.assertEquals(None, repos.lhs_revision_parent("", 0, NoBranchingScheme()))
+        set_branching_scheme(repos, NoBranchingScheme())
+        self.assertEquals(NULL_REVISION, repos.lhs_revision_parent("", 0, NoBranchingScheme()))
 
     def testlhs_revision_parent_first(self):
         repos_url = self.make_client('d', 'dc')
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(NoBranchingScheme())
+        set_branching_scheme(repos, NoBranchingScheme())
         self.build_tree({'dc/adir/afile': "data"})
         self.client_add("dc/adir")
         self.client_commit("dc", "Initial commit")
@@ -1114,7 +1117,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.build_tree({'dc/trunk/adir/afile': "bla"})
         self.client_commit("dc", "Incremental commit")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme())
+        set_branching_scheme(repos, TrunkBranchingScheme())
         mapping = repos.get_mapping()
         self.assertEquals(repos.generate_revision_id(1, "trunk", mapping), \
                 repos.lhs_revision_parent("trunk", 2, mapping))
@@ -1130,7 +1133,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.build_tree({'dc/de/trunk/adir/afile': "bla"})
         self.client_commit("dc", "Change de")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme(1))
+        set_branching_scheme(repos, TrunkBranchingScheme(1))
         mapping = repos.get_mapping()
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping), \
                 repos.lhs_revision_parent("de/trunk", 3, mapping))
@@ -1146,7 +1149,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_copy("dc/py/trunk", "dc/de/trunk")
         self.client_commit("dc", "Copy trunk")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme(1))
+        set_branching_scheme(repos, TrunkBranchingScheme(1))
         mapping = repos.get_mapping()
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping), \
                 repos.lhs_revision_parent("de/trunk", 2, mapping))
@@ -1162,7 +1165,7 @@ class TestSubversionRepositoryWorks(TestCaseWithSubversionRepository):
         self.client_delete("dc/de/trunk/adir")
         self.client_commit("dc", "Another incremental commit")
         repos = Repository.open(repos_url)
-        repos.set_branching_scheme(TrunkBranchingScheme(1))
+        set_branching_scheme(repos, TrunkBranchingScheme(1))
         mapping = repos.get_mapping()
         self.assertEquals(repos.generate_revision_id(1, "py/trunk", mapping), \
                 repos.lhs_revision_parent("de/trunk", 3, mapping))
