@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -19,11 +19,11 @@
 from bzrlib.errors import NotBranchError, BzrError
 
 from bzrlib.tests import TestCase
-from scheme import (ListBranchingScheme, NoBranchingScheme, 
+from mapping3.scheme import (ListBranchingScheme, NoBranchingScheme, 
                     BranchingScheme, TrunkBranchingScheme, 
                     SingleBranchingSchemev0, 
                     SingleBranchingScheme, 
-                    UnknownBranchingScheme,
+                    UnknownBranchingScheme, 
                     parse_list_scheme_text, find_commit_paths, 
                     guess_scheme_from_branch_path, guess_scheme_from_history,
                     guess_scheme_from_path, scheme_from_branch_list)
@@ -274,6 +274,7 @@ class ListScheme(TestCase):
     def test_parse_text_comment(self):
         self.assertEqual(["bla/bloe", "blie"], parse_list_scheme_text("bla/bloe\n# comment\nblie"))
 
+
 class TrunkScheme(TestCase):
     def test_is_branch_empty(self):
         self.assertFalse(TrunkBranchingScheme().is_branch(""))
@@ -591,22 +592,22 @@ class TestGuessBranchingSchemeFromPath(TestCase):
 class TestGuessBranchingSchemeFromHistory(TestCase):
     def test_simple(self):
         scheme = guess_scheme_from_history([
-            ("", {"trunk": ('M', None, None)}, 0)], 1)
+            ({"trunk": ('M', None, None)}, 0, None)], 1)
         self.assertIsInstance(scheme, TrunkBranchingScheme)
         self.assertEqual(0, scheme.level)
 
     def test_simple_with_relpath(self):
         scheme = guess_scheme_from_history([
-            ("", {"trunk": ('M', None, None)}, 0)], 1, 
+            ({"trunk": ('M', None, None)}, 0, None)], 1, 
             relpath="trunk")
         self.assertIsInstance(scheme, TrunkBranchingScheme)
         self.assertEqual(0, scheme.level)
 
     def test_simple_prefer_relpath(self):
         scheme = guess_scheme_from_history([
-            ("", {"trunk": ('M', None, None)}, 1),
-            ("", {"trunk": ('M', None, None)}, 2),
-            ("", {"trunk/bar": ('M', None, None)}, 3),
+            ({"trunk": ('M', None, None)}, 1, None),
+            ({"trunk": ('M', None, None)}, 2, None),
+            ({"trunk/bar": ('M', None, None)}, 3, None),
             ], 3, 
             relpath="trunk/bar")
         self.assertIsInstance(scheme, SingleBranchingScheme)
@@ -614,17 +615,17 @@ class TestGuessBranchingSchemeFromHistory(TestCase):
 
     def test_simple_notwant_single(self):
         scheme = guess_scheme_from_history([
-            ("", {"foo": ('M', None, None)}, 1),
-            ("", {"foo": ('M', None, None)}, 2),
-            ("", {"foo/bar": ('M', None, None)}, 3),
+            ({"foo": ('M', None, None)}, 1, None),
+            ({"foo": ('M', None, None)}, 2, None),
+            ({"foo/bar": ('M', None, None)}, 3, None),
             ], 3)
         self.assertIsInstance(scheme, NoBranchingScheme)
 
     def test_simple_no_bp_common(self):
         scheme = guess_scheme_from_history([
-            ("", {"foo": ('M', None, None)}, 1),
-            ("", {"trunk": ('M', None, None)}, 2),
-            ("", {"trunk": ('M', None, None)}, 3),
+            ({"foo": ('M', None, None)}, 1, None),
+            ({"trunk": ('M', None, None)}, 2, None),
+            ({"trunk": ('M', None, None)}, 3, None),
             ], 3)
         self.assertIsInstance(scheme, TrunkBranchingScheme)
 
@@ -636,6 +637,7 @@ class TestGuessBranchingSchemeFromHistory(TestCase):
         scheme = guess_scheme_from_history([], 0, "trunk")
         self.assertIsInstance(scheme, TrunkBranchingScheme)
 
+
 class SchemeFromBranchListTests(TestCase):
     def test_nobranchingscheme(self):
         self.assertIsInstance(scheme_from_branch_list(["."]), NoBranchingScheme)
