@@ -2,7 +2,7 @@
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -20,7 +20,8 @@ from bzrlib.trace import info, mutter
 import bzrlib.ui as ui
 
 from errors import RebaseNotPresent
-from mapping import parse_revision_id
+import itertools
+from bzrlib.plugins.svn.mapping import parse_revision_id
 
 class UpgradeChangesContent(BzrError):
     """Inconsistency was found upgrading the mapping of a revision."""
@@ -94,22 +95,18 @@ def generate_upgrade_map(new_mapping, revs):
              dictionary.
     """
     rename_map = {}
-    pb = ui.ui_factory.nested_progress_bar()
     # Create a list of revisions that can be renamed during the upgade
-    try:
-        for revid in revs:
-            pb.update('gather revision information', revs.index(revid), len(revs))
-            try:
-                (uuid, bp, rev, mapping) = parse_revision_id(revid)
-            except InvalidRevisionId:
-                # Not a bzr-svn revision, nothing to do
-                continue
-            newrevid = new_mapping.generate_revision_id(uuid, rev, bp)
-            if revid == newrevid:
-                continue
-            rename_map[revid] = newrevid
-    finally:
-        pb.finished()
+    for revid in revs:
+        assert isinstance(revid, str)
+        try:
+            (uuid, bp, rev, mapping) = parse_revision_id(revid)
+        except InvalidRevisionId:
+            # Not a bzr-svn revision, nothing to do
+            continue
+        newrevid = new_mapping.generate_revision_id(uuid, rev, bp)
+        if revid == newrevid:
+            continue
+        rename_map[revid] = newrevid
 
     return rename_map
 
@@ -148,8 +145,13 @@ def create_upgrade_plan(repository, svn_repository, new_mapping,
     from bzrlib.plugins.rebase.rebase import generate_transpose_plan
     check_rebase_version()
 
-    graph = repository.get_revision_graph(revision_id)
-    upgrade_map = generate_upgrade_map(new_mapping, graph.keys())
+    graph = repository.get_graph()
+    if revision_id is None:
+        potential = repository.all_revision_ids()
+    else:
+        potential = itertools.imap(lambda (rev, parents): rev, 
+                graph.iter_ancestry([revision_id]))
+    upgrade_map = generate_upgrade_map(new_mapping, potential)
    
     # Make sure all the required current version revisions are present
     for revid in upgrade_map.values():
@@ -162,7 +164,7 @@ def create_upgrade_plan(repository, svn_repository, new_mapping,
             newrev = repository.get_revision(newrevid)
             check_revision_changed(oldrev, newrev)
 
-    plan = generate_transpose_plan(graph, upgrade_map, 
+    plan = generate_transpose_plan(repository.get_revision_graph(revision_id), upgrade_map, 
       repository.revision_parents,
       lambda revid: create_upgraded_revid(revid, new_mapping.upgrade_suffix))
     def remove_parents((oldrevid, (newrevid, parents))):
