@@ -2,7 +2,7 @@
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
@@ -28,7 +28,7 @@ from bzrlib.lockdir import LockDir
 from bzrlib.osutils import file_kind, fingerprint_file, supports_executable
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
-from bzrlib.tree import RevisionTree
+from bzrlib.revisiontree import RevisionTree
 from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import WorkingTree, WorkingTreeFormat
 
@@ -36,14 +36,13 @@ from branch import SvnBranch
 from commit import _revision_id_to_svk_feature
 from convert import SvnConverter
 from errors import LocalCommitsUnsupported, NoSvnRepositoryPresent
-from mapping import (SVN_PROP_BZR_ANCESTRY, SVN_PROP_BZR_FILEIDS, 
+from bzrlib.plugins.svn.mapping import (SVN_PROP_BZR_ANCESTRY, SVN_PROP_BZR_FILEIDS, 
                      SVN_PROP_BZR_REVISION_ID, SVN_PROP_BZR_REVISION_INFO,
                      generate_revision_metadata)
 from remote import SvnRemoteAccess
 from repository import SvnRepository
 from svk import SVN_PROP_SVK_MERGE, parse_svk_features, serialize_svk_features
-from mapping import escape_svn_path
-from scheme import BranchingScheme
+from bzrlib.plugins.svn.mapping import escape_svn_path
 from transport import (SvnRaTransport, bzr_to_svn_url, create_svn_client,
                        svn_config) 
 from tree import SvnBasisTree
@@ -52,17 +51,23 @@ import os
 import urllib
 
 import svn.core, svn.wc
-from svn.core import SubversionException, Pool
+from svn.core import SubversionException
 
 from errors import NoCheckoutSupport
 from format import get_rich_root_format
 
-class WorkingTreeInconsistent(BzrError):
-    _fmt = """Working copy is in inconsistent state (%(min_revnum)d:%(max_revnum)d)"""
-
-    def __init__(self, min_revnum, max_revnum):
-        self.min_revnum = min_revnum
-        self.max_revnum = max_revnum
+def generate_ignore_list(ignore_map):
+    """Create a list of ignores, ordered by directory.
+    
+    :param ignore_map: Dictionary with paths as keys, patterns as values.
+    :return: list of ignores
+    """
+    ignores = []
+    keys = ignore_map.keys()
+    keys.sort()
+    for k in keys:
+        ignores.append("./" + os.path.join(k.strip("/"), ignore_map[k].strip("/")))
+    return ignores
 
 
 class SvnWorkingTree(WorkingTree):
@@ -73,21 +78,12 @@ class SvnWorkingTree(WorkingTree):
         self.bzrdir = bzrdir
         self._branch = branch
         self.base_revnum = 0
-        self.pool = Pool()
-        self.client_ctx = create_svn_client(self.pool)
+        self.client_ctx = create_svn_client(bzrdir.svn_url)
         self.client_ctx.log_msg_func2 = \
                 svn.client.svn_swig_py_get_commit_log_func
 
         self._get_wc()
         status = svn.wc.revision_status(self.basedir, None, True, None, None)
-        if status.min_rev != status.max_rev:
-            #raise WorkingTreeInconsistent(status.min_rev, status.max_rev)
-            rev = svn.core.svn_opt_revision_t()
-            rev.kind = svn.core.svn_opt_revision_number
-            rev.value.number = status.max_rev
-            assert status.max_rev == svn.client.update(self.basedir, rev,
-                                     True, self.client_ctx, Pool())
-
         self.base_revnum = status.max_rev
         self.base_tree = SvnBasisTree(self)
         self.base_revid = branch.generate_revision_id(self.base_revnum)
@@ -121,6 +117,7 @@ class SvnWorkingTree(WorkingTree):
                 if entry == "":
                     continue
 
+                # Ignore ignores on things that aren't directories
                 if entries[entry].kind != svn.core.svn_node_dir:
                     continue
 
@@ -728,13 +725,13 @@ class SvnCheckout(BzrDir):
         # Open related remote repository + branch
         wc = svn.wc.adm_open3(None, self.local_path, False, 0, None)
         try:
-            svn_url = svn.wc.entry(self.local_path, wc, True).url
+            self.svn_url = svn.wc.entry(self.local_path, wc, True).url
         finally:
             svn.wc.adm_close(wc)
 
-        remote_transport = SvnRaTransport(svn_url)
-        self.remote_bzrdir = SvnRemoteAccess(remote_transport)
-        self.svn_root_transport = remote_transport.clone_root()
+        self.remote_transport = SvnRaTransport(self.svn_url)
+        self.remote_bzrdir = SvnRemoteAccess(self.remote_transport)
+        self.svn_root_transport = self.remote_transport.clone_root()
         self.root_transport = self.transport = transport
         
     def clone(self, path, revision_id=None, force_new_repo=False):
@@ -788,7 +785,7 @@ class SvnCheckout(BzrDir):
         repos = self._find_repository()
 
         try:
-            branch = SvnBranch(self.svn_root_transport.base, repos, 
+            branch = SvnBranch(self.remote_transport.base, repos, 
                                self.remote_bzrdir.branch_path)
         except SubversionException, (_, num):
             if num == svn.core.SVN_ERR_WC_NOT_DIRECTORY:
